"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.model = void 0;
const controller_1 = require("@stargate-system/controller");
const model_1 = require("@stargate-system/model");
let heaterSwitch;
let pumpSwitch;
let valvePosition;
const pump = {
    switchOn: () => pumpSwitch === null || pumpSwitch === void 0 ? void 0 : pumpSwitch.setValue(true),
    switchOff: () => pumpSwitch === null || pumpSwitch === void 0 ? void 0 : pumpSwitch.setValue(false),
    isReady: () => pumpSwitch !== undefined
};
const heater = {
    switchOn: () => heaterSwitch === null || heaterSwitch === void 0 ? void 0 : heaterSwitch.setValue(true),
    switchOff: () => heaterSwitch === null || heaterSwitch === void 0 ? void 0 : heaterSwitch.setValue(false),
    isReady: () => heaterSwitch !== undefined
};
const valve = {
    setPosition: (value) => valvePosition === null || valvePosition === void 0 ? void 0 : valvePosition.setValue(value),
    isReady: () => valvePosition !== undefined
};
const thermometer = {
    temperature: undefined,
    onTemperatureChange: () => { },
    isReady: () => thermometer.temperature !== undefined
};
const logMessage = (message) => {
    systemModel.serverStorage.append('heating.txt', `${(new Date()).toLocaleString()} > ${message}\n`, './logs');
};
exports.model = {
    heater,
    pump,
    valve,
    thermometer,
    onModelChange: () => { },
    logMessage
};
const systemModel = (0, controller_1.getSystemModel)();
const initDeviceModel = (deviceModel, onDeviceUp, onDeviceDown) => {
    if (deviceModel && deviceModel.state.value === model_1.DeviceState.up) {
        onDeviceUp();
        exports.model.onModelChange();
        deviceModel.state.subscribe(() => {
            if (deviceModel.state.value !== model_1.DeviceState.up) {
                onDeviceDown();
                exports.model.onModelChange();
            }
        });
    }
    else {
        onDeviceDown();
        exports.model.onModelChange();
    }
};
new model_1.DeviceSubscription(systemModel, (device) => device.name.value === 'Heater', (device) => initDeviceModel(device, () => { var _a; return heaterSwitch = (_a = device === null || device === void 0 ? void 0 : device.gateValues.values[0]) === null || _a === void 0 ? void 0 : _a.gateValue; }, () => heaterSwitch = undefined));
new model_1.DeviceSubscription(systemModel, (device) => device.manifest.uuid === '0be8e7f0-2c70-4cfd-a10e-36ba70222f3c', (device) => initDeviceModel(device, () => { var _a; return pumpSwitch = (_a = device === null || device === void 0 ? void 0 : device.gateValues.values[0]) === null || _a === void 0 ? void 0 : _a.gateValue; }, () => pumpSwitch = undefined));
new model_1.DeviceSubscription(systemModel, (device) => device.info.value === 'valve', (device) => initDeviceModel(device, () => { var _a; return valvePosition = (_a = device === null || device === void 0 ? void 0 : device.gateValues.values[0]) === null || _a === void 0 ? void 0 : _a.gateValue; }, () => valvePosition = undefined));
new model_1.DeviceSubscription(systemModel, (device) => (device.group.value === 'Living Room') && (device.name.value === 'Weather Station'), (device) => initDeviceModel(device, () => {
    const temperature = device === null || device === void 0 ? void 0 : device.gateValues.find((value) => value.gateValue.valueName === 'Temperature');
    if (temperature) {
        thermometer.temperature = temperature.modelValue.value;
        thermometer.onTemperatureChange();
        temperature.modelValue.subscribe(() => {
            thermometer.temperature = temperature.modelValue.value;
            thermometer.onTemperatureChange();
        });
    }
    else {
        thermometer.temperature = undefined;
    }
}, () => thermometer.temperature = undefined));
