"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const device_1 = require("@stargate-system/device");
const controller_1 = require("@stargate-system/controller");
const model_1 = require("@stargate-system/model");
device_1.GateDevice.setName('Weather Station');
const temperature = device_1.GateDevice.factory.createFloat(device_1.Directions.output);
temperature.valueName = 'Temperature';
const humidity = device_1.GateDevice.factory.createInteger(device_1.Directions.output);
humidity.valueName = 'Humidity (%)';
humidity.setRange([0, 100]);
device_1.GateDevice.start();
const initDevice = (device) => {
    if (device) {
        device.gateValues.values[0].modelValue.subscribe(() => { });
    }
};
const systemModel = (0, controller_1.getSystemModel)();
const model = {
    temperature: 14,
    externalTemperature: 14
};
const heater = new model_1.DeviceSubscription(systemModel, (device) => device.name.value === 'Heater', initDevice);
const pump = new model_1.DeviceSubscription(systemModel, (device) => device.manifest.uuid === '0be8e7f0-2c70-4cfd-a10e-36ba70222f3c', initDevice);
const valve = new model_1.DeviceSubscription(systemModel, (device) => device.info.value === 'valve', initDevice);
const isHeaterActive = () => {
    var _a, _b;
    return (((_a = heater.deviceModel) === null || _a === void 0 ? void 0 : _a.state.value) === model_1.DeviceState.up) && ((_b = heater.deviceModel) === null || _b === void 0 ? void 0 : _b.gateValues.values[0].gateValue.value);
};
const isPumpActive = () => {
    var _a, _b;
    return (((_a = pump.deviceModel) === null || _a === void 0 ? void 0 : _a.state.value) === model_1.DeviceState.up) && ((_b = pump.deviceModel) === null || _b === void 0 ? void 0 : _b.gateValues.values[0].gateValue.value);
};
const getValveState = () => {
    var _a;
    if (((_a = valve.deviceModel) === null || _a === void 0 ? void 0 : _a.state.value) !== model_1.DeviceState.up) {
        return 0;
    }
    return valve.deviceModel.gateValues.values[0].gateValue.value;
};
const recalculate = () => {
    calculateTemperature();
    calculateHumidity();
};
const calculateTemperature = () => {
    let heatIn = 0;
    const valveState = getValveState();
    if (isHeaterActive() && isPumpActive() && (valveState > 0)) {
        heatIn = valveState / 200;
    }
    const heatOut = (model.temperature - model.externalTemperature) / 40;
    model.temperature += heatIn - heatOut;
    temperature.setValue(Math.round(model.temperature * 10) / 10);
};
const calculateHumidity = () => {
    let calculated = 82 - (model.temperature - model.externalTemperature) * 3;
    if (calculated > 100) {
        calculated = 100;
    }
    else if (calculated < 0) {
        calculated = 0;
    }
    humidity.setValue(Math.round(calculated));
};
setInterval(recalculate, 500);
