"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initConnectionService = void 0;
const core_1 = require("@stargate-system/core");
const LocalDeviceConnector_1 = require("./device/LocalDeviceConnector");
const LocalControllerConnector_1 = require("./controller/LocalControllerConnector");
const ws_1 = require("ws");
const Router_1 = __importDefault(require("./Router"));
const config_1 = __importDefault(require("../config"));
const initConnectionService = () => {
    const server = new ws_1.WebSocketServer({ port: config_1.default.connectionPort });
    server.on('connection', (socket, request) => {
        console.log("New connection from " + request.socket.remoteAddress);
        socket.on('error', console.log);
        const socketWrapper = {
            send: socket.send.bind(socket),
            close: () => {
                socket.removeAllListeners();
                socket.close();
            },
            setOnClose: (callback) => socket.onclose = callback,
            setOnMessage: (callback) => socket.on('message', (ev) => callback(ev.toString()))
        };
        const connection = new core_1.DefaultConnection(true);
        connection.setConnected(socketWrapper);
        const stateListenerKey = connection.addStateChangeListener((state) => {
            if (state === core_1.ConnectionState.closed) {
                console.log("Connection from " + request.socket.remoteAddress + " failed");
            }
        });
        connection.functionalHandler.createQuery(core_1.Keywords.type).then((response) => {
            console.log("Connection type for " + request.socket.remoteAddress + " - " + response);
            switch (response) {
                case core_1.ConnectionType.device:
                    new LocalDeviceConnector_1.LocalDeviceConnector(connection);
                    break;
                case core_1.ConnectionType.controller:
                    const controllerConnector = new LocalControllerConnector_1.LocalControllerConnector(connection);
                    Router_1.default.addController(controllerConnector);
                    break;
                default:
                    connection.close();
            }
            connection.removeStateChangeListener(stateListenerKey);
        }).catch(() => {
            connection.close();
        });
    });
};
exports.initConnectionService = initConnectionService;
