"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@stargate-system/core");
const Router_1 = __importDefault(require("../Router"));
const DeviceContext_1 = __importDefault(require("../device/DeviceContext"));
const controllerRegistry = new core_1.Registry();
const addController = (controller) => __awaiter(void 0, void 0, void 0, function* () {
    controller.id = controllerRegistry.add(controller);
    controller.onDisconnect = () => controllerDisconnected(controller);
    controller.onValueMessage = DeviceContext_1.default.forwardValueMessage;
    const systemImage = yield Router_1.default.systemRepository.getSystemImage();
    const activeDeviceIds = DeviceContext_1.default.getActiveDeviceIds();
    controller.sendJoinData(systemImage, activeDeviceIds);
    controller.onSubscribed = (ids) => {
        DeviceContext_1.default.forwardSubscribed(ids, controller);
    };
    controller.onUnsubscribed = (ids) => {
        DeviceContext_1.default.forwardUnsubscribed(ids, controller);
    };
    controller.onDeviceEvent = (event, data) => {
        switch (event) {
            case core_1.EventName.deviceRemoved:
                if (data[0]) {
                    const [id] = data;
                    if (DeviceContext_1.default.deviceRegistry.getByKey(id)) {
                        console.log('Cancelled removing device: device is active');
                    }
                    else {
                        Router_1.default.systemRepository.removeDevice(id);
                        Router_1.default.serverStorage.remove(id);
                        DeviceContext_1.default.notifyPipes(core_1.EventName.deviceRemoved, id);
                        forwardDeviceEvent(core_1.EventName.deviceRemoved, [id]);
                    }
                }
                break;
            case core_1.EventName.deviceRenamed:
                if (data[0] && data[1]) {
                    const [id, newName] = data;
                    Router_1.default.systemRepository.renameDevice(id, newName);
                    forwardDeviceEvent(core_1.EventName.deviceRenamed, [id, newName]);
                }
                break;
            case core_1.EventName.addedToGroup:
                if (data.length > 1) {
                    const groupName = data[0];
                    const deviceIds = data.slice(1);
                    Router_1.default.systemRepository.addDevicesToGroup(groupName, deviceIds);
                    forwardDeviceEvent(core_1.EventName.addedToGroup, [groupName !== null && groupName !== void 0 ? groupName : '', ...deviceIds]);
                }
                break;
            case core_1.EventName.deviceModified:
                if (data[0]) {
                    try {
                        const manifest = JSON.parse(data[0]);
                        Router_1.default.systemRepository.overwriteDevice(manifest).then((success) => {
                            if (success) {
                                forwardDeviceEvent(core_1.EventName.deviceModified, data);
                            }
                            else {
                                console.log('Failed to modify device', data);
                            }
                        });
                    }
                    catch (err) {
                        console.log('On device modified', err);
                    }
                }
                break;
            default:
                console.log('Unknown device event: ' + event);
        }
    };
    controller.onPipeEvent = (event, data) => {
        if (data[0] && data[1]) {
            const pipe = data;
            switch (event) {
                case core_1.EventName.pipeCreated:
                    Router_1.default.systemRepository.createPipe(pipe).then((success) => {
                        if (success) {
                            DeviceContext_1.default.addPipe(pipe);
                            forwardPipeEvent(core_1.EventName.pipeCreated, pipe);
                        }
                        else {
                            console.log('Failed to create pipe', data);
                        }
                    });
                    break;
                case core_1.EventName.pipeRemoved:
                    DeviceContext_1.default.removePipe(pipe);
                    break;
                default:
                    console.log('Unknown pipe event: ' + event);
            }
        }
    };
    console.log('Connected controller ' + controller.id);
});
const controllerDisconnected = (controller) => {
    if (controller.id) {
        controllerRegistry.remove(controller.id);
        DeviceContext_1.default.unsubscribeConsumer(controller.id);
        console.log('Disconnected controller ' + controller.id);
    }
};
const forwardDeviceEvent = (eventName, data) => {
    controllerRegistry.getValues()
        .forEach((controller) => controller.sendDeviceEvent(eventName, data));
};
const forwardPipeEvent = (eventName, data) => {
    controllerRegistry.getValues()
        .forEach((controller) => controller.sendPipeEvent(eventName, data));
};
const forwardValueMessage = (valueMessage) => {
    controllerRegistry.getValues().forEach((controller) => {
        controller.sendValueMessage(valueMessage);
    });
};
const ControllerContext = {
    controllerRegistry,
    addController,
    forwardDeviceEvent,
    forwardPipeEvent,
    forwardValueMessage
};
exports.default = ControllerContext;
