"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initDeviceContext = void 0;
const core_1 = require("@stargate-system/core");
const Pipe_1 = require("./Pipe");
const Router_1 = __importDefault(require("../Router"));
const deviceRegistry = new core_1.Registry();
let pipes = [];
const initDeviceContext = () => __awaiter(void 0, void 0, void 0, function* () {
    const image = yield Router_1.default.systemRepository.getSystemImage();
    if (image.pipes) {
        pipes = image.pipes.map((pipe) => new Pipe_1.Pipe(pipe[0], pipe[1]));
    }
});
exports.initDeviceContext = initDeviceContext;
const forwardValueMessage = (valueMessage) => {
    valueMessage.forEach((message) => {
        const [deviceId, valueId] = core_1.AddressMapper.extractTargetId(message[0]);
        const device = deviceRegistry.getByKey(deviceId);
        if (device) {
            device.sendValue([valueId, message[1]]);
        }
    });
};
const handleSubscription = (subscribe, ids, source) => {
    const devicesMap = new Map();
    ids.forEach((idWithParent) => {
        const [parentId, valueId] = core_1.AddressMapper.extractTargetId(idWithParent);
        let valueIds = devicesMap.get(parentId);
        if (valueIds) {
            valueIds.push(valueId);
        }
        else {
            devicesMap.set(parentId, [valueId]);
        }
    });
    devicesMap.forEach((valueIds, deviceId) => {
        const targetDevice = deviceRegistry.getByKey(deviceId);
        if (targetDevice) {
            if (subscribe) {
                valueIds.forEach((id) => {
                    targetDevice.subscribe(id, source);
                });
            }
            else {
                valueIds.forEach((id) => {
                    targetDevice.unsubscribe(id, source.id);
                });
            }
        }
    });
};
const forwardSubscribed = (ids, source) => {
    handleSubscription(true, ids, source);
};
const forwardUnsubscribed = (ids, source) => {
    handleSubscription(false, ids, source);
};
const getActiveDeviceIds = () => {
    return deviceRegistry.getValues().map((device) => device.id);
};
const unsubscribeConsumer = (consumerId) => {
    deviceRegistry.getValues().forEach((device) => device.unsubscribeConsumer(consumerId));
};
const addPipe = (pipe) => {
    pipes.push(new Pipe_1.Pipe(pipe[0], pipe[1]));
};
const removePipe = (pipe) => {
    const removedId = pipe[0] + pipe[1];
    let removedPipe;
    pipes = pipes.filter((storedPipe) => {
        const isRemoved = storedPipe.id === removedId;
        if (isRemoved) {
            removedPipe = storedPipe;
        }
        return !isRemoved;
    });
    if (removedPipe) {
        removedPipe.remove();
    }
};
const notifyPipes = (event, device) => {
    pipes.forEach((pipe) => {
        pipe.handleDeviceEvent(event, device);
    });
};
const DeviceContext = {
    deviceRegistry,
    forwardValueMessage,
    forwardSubscribed,
    forwardUnsubscribed,
    getActiveDeviceIds,
    unsubscribeConsumer,
    addPipe,
    removePipe,
    notifyPipes
};
exports.default = DeviceContext;
