"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalDeviceConnector = void 0;
const core_1 = require("@stargate-system/core");
const Router_1 = __importDefault(require("../Router"));
const Device_1 = require("./Device");
class LocalDeviceConnector {
    constructor(connection) {
        this.performHandshake = () => __awaiter(this, void 0, void 0, function* () {
            const functionalHandler = this._connection.functionalHandler;
            if (this._connection.state === core_1.ConnectionState.connected) {
                const manifestString = yield functionalHandler
                    .createQuery(core_1.Keywords.manifest)
                    .catch(() => this._connection.close());
                if (manifestString !== undefined) {
                    try {
                        const manifest = JSON.parse(manifestString);
                        if (manifest.id === undefined) {
                            this._manifest = yield Router_1.default.systemRepository.createDevice(manifest);
                            functionalHandler.sendCommand(core_1.Keywords.assignedId, [this._manifest.uuid]);
                        }
                        else {
                            manifest.uuid = manifest.id;
                            this._manifest = yield Router_1.default.systemRepository.updateDevice(manifest);
                        }
                        // @ts-ignore
                        this._id = this._manifest.id;
                        new Device_1.Device(this);
                    }
                    catch (err) {
                        console.log("Failed connection (on handshake) for manifest: " + manifestString, err);
                        this._connection.close();
                    }
                }
                else {
                    this._connection.close();
                }
            }
        });
        this._connection = connection;
        this.performHandshake();
    }
    get id() {
        return this._id;
    }
    get manifest() {
        return this._manifest;
    }
    get connection() {
        return this._connection;
    }
}
exports.LocalDeviceConnector = LocalDeviceConnector;
