"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pipe = void 0;
const core_1 = require("@stargate-system/core");
const Router_1 = __importDefault(require("../Router"));
const DeviceContext_1 = __importDefault(require("./DeviceContext"));
const ControllerContext_1 = __importDefault(require("../controller/ControllerContext"));
class Pipe {
    constructor(sourceId, targetId) {
        this.handleDeviceEvent = (event, device) => {
            var _a;
            switch (event) {
                case core_1.EventName.deviceConnected:
                    if (!(this.sourceDevice && this.targetDevice)) {
                        const deviceId = device.id;
                        if (this.sourceDeviceId === deviceId) {
                            this.sourceDevice = device;
                        }
                        else if (this.targetDeviceId === deviceId) {
                            this.targetDevice = device;
                        }
                        this.activatePipe();
                    }
                    break;
                case core_1.EventName.deviceDisconnected:
                    const disconnectedId = device;
                    const wasSubscribed = !!this.sourceDevice && !!this.targetDevice;
                    if (this.sourceDeviceId === disconnectedId) {
                        this.sourceDevice = undefined;
                    }
                    else if (this.targetDeviceId === disconnectedId) {
                        this.targetDevice = undefined;
                        if (wasSubscribed) {
                            (_a = this.sourceDevice) === null || _a === void 0 ? void 0 : _a.unsubscribe(this.sourceValueId, this._id);
                        }
                    }
                    break;
                case core_1.EventName.deviceRemoved:
                    const removedId = device;
                    if (this.sourceDeviceId === removedId || this.targetDeviceId === removedId) {
                        DeviceContext_1.default.removePipe(this.pipe);
                    }
                    break;
            }
        };
        this.sendValueMessage = (valueMessage) => {
            if (this.targetDevice) {
                this.targetDevice.sendValue([this.targetValueId, valueMessage[0][1]]);
            }
        };
        this.remove = () => {
            if (this.sourceDevice && this.targetDevice) {
                this.sourceDevice.unsubscribe(this.sourceValueId, this._id);
            }
            Router_1.default.systemRepository.removePipe(this.pipe);
            ControllerContext_1.default.forwardPipeEvent(core_1.EventName.pipeRemoved, this.pipe);
        };
        this.areTypesCompatible = (type1, type2) => {
            if (type1 === type2) {
                return true;
            }
            if (type1 === core_1.ValueTypes.integer || type1 === core_1.ValueTypes.float) {
                return type2 === core_1.ValueTypes.integer || type2 === core_1.ValueTypes.float;
            }
            return false;
        };
        this.activatePipe = () => {
            var _a, _b;
            if (this.sourceDevice && this.targetDevice) {
                const sourceValueType = (_a = this.sourceDevice.manifest.values
                    .find((value) => value.id === this.sourceValueId)) === null || _a === void 0 ? void 0 : _a.type;
                const targetValueType = (_b = this.targetDevice.manifest.values
                    .find((value) => value.id === this.targetValueId)) === null || _b === void 0 ? void 0 : _b.type;
                if (sourceValueType && targetValueType && this.areTypesCompatible(sourceValueType, targetValueType)) {
                    this.sourceDevice.subscribe(this.sourceValueId, this);
                }
                else {
                    console.log('Dropping pipe ' + this.pipe[0] + ' -> ' + this.pipe[1] + ' - values incompatible');
                    DeviceContext_1.default.removePipe(this.pipe);
                }
            }
        };
        this._id = sourceId + targetId;
        this.pipe = [sourceId, targetId];
        const [sourceDeviceId, sourceValueId] = core_1.AddressMapper.extractTargetId(sourceId);
        this.sourceDeviceId = sourceDeviceId;
        this.sourceValueId = sourceValueId;
        const [targetDeviceId, targetValueId] = core_1.AddressMapper.extractTargetId(targetId);
        this.targetDeviceId = targetDeviceId;
        this.targetValueId = targetValueId;
        DeviceContext_1.default.deviceRegistry.getValues().forEach((device) => {
            if (device.id === this.sourceDeviceId) {
                this.sourceDevice = device;
            }
            else if (device.id === this.targetDeviceId) {
                this.targetDevice = device;
            }
        });
        setTimeout(this.activatePipe, 0);
    }
    get id() {
        return this._id;
    }
}
exports.Pipe = Pipe;
