"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const ROOT_DIR = './ServerStorage';
const get = (directory, key) => {
    return new Promise((resolve) => {
        const path = ROOT_DIR + '/' + directory + '/' + key;
        fs_1.default.readFile(path, (err, data) => {
            if (err) {
                console.log('On getting from ServerStorage', err);
                resolve(undefined);
            }
            else {
                resolve(data.toString());
            }
        });
    });
};
const set = (directory, key, value) => {
    if (directory.length && key.length && value.length) {
        const path = ROOT_DIR + '/' + directory;
        fs_1.default.promises.mkdir(path, { recursive: true }).then(() => {
            fs_1.default.writeFile(path + '/' + key, value, (err) => {
                if (err) {
                    console.log('On setting to ServerStorage', err);
                }
            });
        }).catch(console.log);
    }
};
const append = (directory, key, value) => {
    if (directory.length && key.length && value.length) {
        const path = ROOT_DIR + '/' + directory + '/' + key;
        const file = fs_1.default.createWriteStream(path, { flags: 'a' });
        file.on('error', (err) => {
            fs_1.default.lstat(path, (statErr, stats) => {
                if (statErr && !stats) {
                    set(directory, key, value);
                }
                else {
                    console.log('On appending to ServerStorage', err);
                }
            });
        });
        file.on('ready', () => {
            file.write(value);
            file.close();
        });
    }
};
const remove = (directory, key, logError) => {
    if (key === undefined) {
        fs_1.default.rm(ROOT_DIR + '/' + directory, { recursive: true }, (err) => {
            if (err && logError) {
                console.log('On removing directory from ServerStorage', err);
            }
        });
    }
    else {
        fs_1.default.rm(ROOT_DIR + '/' + directory + '/' + key, (err) => {
            if (err && logError) {
                console.log('On removing key from ServerStorage', err);
            }
        });
    }
};
const BasicServerStorage = {
    get,
    set,
    append,
    remove
};
exports.default = BasicServerStorage;
