"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
let systemImage;
let saveTimeout;
const generateId = () => {
    if (systemImage.devices.length === 0) {
        return '0';
    }
    const ids = systemImage.devices
        .map((device) => Number.parseInt(device.id))
        .sort((a, b) => a - b);
    if (ids[0] > 0) {
        return '0';
    }
    let previousId = undefined;
    for (const id of ids) {
        if (previousId === undefined) {
            previousId = id;
        }
        else {
            if (id - previousId > 1) {
                break;
            }
            else {
                previousId = id;
            }
        }
    }
    // @ts-ignore
    return (previousId + 1).toString();
};
const BasicSystemRepository = {
    getSystemImage: () => __awaiter(void 0, void 0, void 0, function* () {
        return systemImage;
    }),
    createDevice: (manifest) => __awaiter(void 0, void 0, void 0, function* () {
        const validManifest = manifest;
        validManifest.uuid = crypto_1.default.randomUUID();
        validManifest.id = generateId();
        systemImage.devices.push(validManifest);
        saveRepository();
        return validManifest;
    }),
    updateDevice: (manifest) => __awaiter(void 0, void 0, void 0, function* () {
        const index = systemImage.devices.findIndex((storedManifest) => storedManifest.uuid === manifest.uuid);
        if (index !== -1) {
            const storedDevice = systemImage.devices[index];
            manifest.id = storedDevice.id;
            manifest.deviceName = storedDevice.deviceName;
            manifest.group = storedDevice.group;
            storedDevice.values.forEach((storedValue) => {
                const newValue = manifest.values.find((value) => storedValue.id === value.id);
                if (newValue && storedValue.type === newValue.type) {
                    newValue.visibility = storedValue.visibility;
                }
            });
            systemImage.devices[index] = manifest;
        }
        else {
            manifest.id = generateId();
            systemImage.devices.push(manifest);
        }
        saveRepository();
        return manifest;
    }),
    overwriteDevice: (manifest) => __awaiter(void 0, void 0, void 0, function* () {
        const index = systemImage.devices.findIndex((storedManifest) => storedManifest.uuid === manifest.uuid);
        if (index !== -1) {
            systemImage.devices[index] = manifest;
            saveRepository();
            return true;
        }
        return false;
    }),
    removeDevice: (id) => {
        systemImage.devices = systemImage.devices.filter((device) => device.id !== id);
        saveRepository();
    },
    renameDevice: (id, newName) => {
        const device = systemImage.devices.find((manifest) => manifest.id === id);
        if (device) {
            device.deviceName = newName;
        }
        saveRepository();
    },
    createPipe: (pipe) => __awaiter(void 0, void 0, void 0, function* () {
        if (!systemImage.pipes.find((storedPipe) => storedPipe[0] === pipe[0] && storedPipe[1] === pipe[1])) {
            systemImage.pipes.push(pipe);
            saveRepository();
            return true;
        }
        return false;
    }),
    removePipe: (pipe) => {
        systemImage.pipes = systemImage.pipes.filter((storedPipe) => storedPipe[0] !== pipe[0] || storedPipe[1] !== pipe[1]);
        saveRepository();
    },
    addDevicesToGroup: (groupName, deviceIds) => {
        deviceIds.forEach((id) => {
            const device = systemImage.devices.find((manifest) => manifest.id === id);
            const groupValue = (groupName !== undefined && groupName.length > 0) ? groupName : undefined;
            if (device) {
                device.group = groupValue;
            }
        });
        saveRepository();
    }
};
const getBasicRepository = () => {
    if (!systemImage) {
        try {
            const systemImageFile = fs_1.default.readFileSync('systemImage.json');
            systemImage = JSON.parse(systemImageFile.toString());
        }
        catch (err) {
            systemImage = { devices: [], pipes: [] };
        }
    }
    return BasicSystemRepository;
};
const saveRepository = () => {
    if (saveTimeout === undefined) {
        saveTimeout = setTimeout(() => {
            saveTimeout = undefined;
            fs_1.default.writeFile('systemImage.json', JSON.stringify(systemImage), (err) => {
                if (err) {
                    console.log('On saving repository', err);
                }
            });
        }, 1000);
    }
};
exports.default = getBasicRepository;
