const {GateDevice, Directions, ValueVisibility, ConnectionState} = require('@stargate-system/device');

GateDevice.setName('Time relay');
GateDevice.setGroup('Utils');

const input = GateDevice.factory.createBoolean(Directions.input);
input.valueName = 'Input';
input.visibility = ValueVisibility.hidden;

const output = GateDevice.factory.createBoolean(Directions.output);
output.valueName = 'Output';
output.visibility = ValueVisibility.hidden;

const delay = GateDevice.factory.createInteger(Directions.input);
delay.valueName = 'Delay';
delay.setMinimum(1);
delay.onRemoteUpdate = () => {
    GateDevice.ServerStorage.set('delay', delay.toString());
}

GateDevice.state.onStateChange = (state) => {
    if (state === ConnectionState.ready) {
        GateDevice.ServerStorage.get('delay').then((response) => {
            const value = Number.parseInt(response);
            if (Number.isInteger(value)) {
                delay.setValue(value);
            }
            GateDevice.state.onStateChange = undefined;
        })
    }
}

let timeout;

const resetDelay = () => {
    if (timeout) {
        clearTimeout(timeout);
        timeout = undefined;
    }
};

const activateDelay = () => {
    resetDelay();
    timeout = setTimeout(() => {
        output.setValue(false);
        timeout = undefined;
    }, delay.value * 1000);
};

input.onRemoteUpdate = () => {
    if (input.value) {
        resetDelay();
        output.setValue(true);
    } else {
        activateDelay();
    }
}

GateDevice.start();
